#ifndef _TTCAN_DRIVER_API_H
#define _TTCAN_DRIVER_API_H

#define TTCANDEV                    0xA462
#define CAN_SIMPLE_DEVICE_VERSION   0
#define CAN_DEVTYPE_NONE            0

/////////////////////////////////////////////////////////////////////////////
// I/O extended codes

#define CTL_CODE(DeviceType, Function, Method, Access) (                 \
    ((DeviceType) << 16) | ((Access) << 14) | ((Function) << 2) | (Method) )

#define METHOD_BUFFERED                 0
#define METHOD_IN_DIRECT                1
#define METHOD_OUT_DIRECT               2
#define METHOD_NEITHER                  3

#define FILE_ANY_ACCESS                 0
#define FILE_READ_ACCESS                ( 0x0001 )    // file & pipe
#define FILE_WRITE_ACCESS               ( 0x0002 )    // file & pipe

/////////////////////////////////////////////////////////////////////////////
// Interrupts
#define INT_MAIN          		0x100C
#define INT_MAIN_MASK	0x1010

#define INT_HDAT			0x01
#define INT_QDAT			0x02
#define INT_CAN1			0x10
#define INT_CAN2			0x20
#define INT_TIM_CAN1	0x40
#define INT_TIM_CAN2	0x80

#define INT_RX0IE		0x01
#define INT_RX1IE		0x02
#define INT_TX0IE		0x04
#define INT_TX1IE		0x08
#define INT_TX2IE		0x10
#define INT_ERRIE		0x20
#define INT_WAKIE		0x40
#define INT_MERRE	0x80

#define HDAT			1
#define QDAT			2
#define TIM_CAN1	3
#define TIM_CAN2	4
#define RX0IE			5
#define RX1IE			6
#define TX0IE			7
#define TX1IE			8
#define TX2IE			9
#define ERRIE			10
#define WAKIE			11
#define MERRE		12


#define IOCTL_TTCANDEV_REGISTER_INTERRUPT \
	CTL_CODE(TTCANDEV, 0x800, METHOD_BUFFERED, FILE_ANY_ACCESS)
	
#define IOCTL_TTCANDEV_DISABLE_INTERRUPT \
	CTL_CODE(TTCANDEV, 0x801, METHOD_BUFFERED, FILE_ANY_ACCESS)
	
#define IOCTL_TTCANDEV_GET_DEVICE_VER \
  CTL_CODE(TTCANDEV, 0x900, METHOD_BUFFERED, FILE_READ_ACCESS)

#define IOCTL_TTCANDEV_INT_FLAGS \
  CTL_CODE(TTCANDEV, 0x901, METHOD_BUFFERED, FILE_READ_ACCESS)
  
#define IOCTL_TTCANDEV_GET_NBLOCK_RAW_DMA \
  CTL_CODE(TTCANDEV, 0x903, METHOD_BUFFERED, FILE_READ_ACCESS)
 
#define IOCTL_TTCANDEV_GET_STATUS \
  CTL_CODE(TTCANDEV, 0x904, METHOD_BUFFERED, FILE_ANY_ACCESS)
  
#define IOCTL_TTCANDEV_READ_DMA_DATA_BLOCKS \
  CTL_CODE(TTCANDEV, 0x905, METHOD_BUFFERED, FILE_ANY_ACCESS)
  
#define IOCTL_TTCANDEV_DISABLE_CANONICAL_DMA \
  CTL_CODE(TTCANDEV, 0x907, METHOD_BUFFERED, FILE_ANY_ACCESS)
  
#define IOCTL_TTCANDEV_READ_DMA_CH_DATA_BLOCKS \
  CTL_CODE(TTCANDEV, 0x908, METHOD_BUFFERED, FILE_ANY_ACCESS)
  
#define IOCTL_TTCANDEV_ENABLE_CANONICAL_DMA \
  CTL_CODE(TTCANDEV, 0x909, METHOD_BUFFERED, FILE_ANY_ACCESS)
    
#define IOCTL_TTCANDEV_GET_RESOURCE_INFO \
  CTL_CODE(TTCANDEV, 0x950, METHOD_BUFFERED, FILE_ANY_ACCESS)

#define IOCTL_TTCANDEV_CLEAR_DMA \
  CTL_CODE(TTCANDEV, 0x953, METHOD_BUFFERED, FILE_ANY_ACCESS)

#define IOCTL_TTCANDEV_READ_BAR \
  CTL_CODE(TTCANDEV, 0x954, METHOD_BUFFERED, FILE_READ_ACCESS)

#define IOCTL_TTCANDEV_WRITE_BAR \
  CTL_CODE(TTCANDEV, 0x955, METHOD_BUFFERED, FILE_WRITE_ACCESS)
  
#define IOCTL_TTCANDEV_DMA_COUNT \
  CTL_CODE(TTCANDEV, 0x966, METHOD_BUFFERED, FILE_ANY_ACCESS)
  
#define IOCTL_TTCANDEV_READ_DMA_DATABUF \
  CTL_CODE(TTCANDEV, 0x967, METHOD_BUFFERED, FILE_ANY_ACCESS)
  
#define IOCTL_TTCANDEV_READ_DMA_STATUS \
  CTL_CODE(TTCANDEV, 0x968, METHOD_BUFFERED, FILE_ANY_ACCESS)  

#define IOCTL_TTCANDEV_GET_PROPERTY \
  CTL_CODE(TTCANDEV, 0x96A, METHOD_BUFFERED, FILE_ANY_ACCESS)

#define IOCTL_TTCANDEV_READ_DATABUF \
  CTL_CODE(TTCANDEV, 0x96C, METHOD_OUT_DIRECT, FILE_READ_ACCESS)

#define IOCTL_TTCANDEV_WRITE_DATABUF \
  CTL_CODE(TTCANDEV, 0x96D, METHOD_BUFFERED, FILE_WRITE_ACCESS)
  
#define IOCTL_TTCANDEV_WRITE_REG  \
  CTL_CODE(TTCANDEV, 0x97D, METHOD_BUFFERED, FILE_ANY_ACCESS)
  
#define IOCTL_TTCANDEV_READ_REG    \
  CTL_CODE(TTCANDEV, 0x97E, METHOD_BUFFERED, FILE_ANY_ACCESS)
 

/////////////////////////////////////////////////////////////////////////////
//Structs

//IOCTL_TTCANDEV_GET_DEVICE_VER
typedef struct _TTCAN_GET_REV_OUT{
  UINT32 m_nRev;
} TTCAN_GET_REV_OUT;

//IOCTL_TTCANDEV_GET_NBLOCK_RAW_DMA
typedef struct _TTCAN_GET_NBLOCK_DMA_OUT{
  UINT32 m_nValue;
} TTCAN_GET_NBLOCK_DMA_OUT;

// IOCTL_TTCANDEV_READ_DMA_DATA_BLOCKS
typedef struct _TTCAN_READ_DMA_DATA_BLOCKS_IN{
  UINT32 m_nBlocks;        // Size (in bytes)
} TTCAN_READ_DMA_DATA_BLOCKS_IN;

// IOCTL_TTCANDEV_READ_DMA_CH_DATA_BLOCKS
typedef struct _TTCAN_READ_DMA_CH_DATA_BLOCKS_IN{
  UINT32 m_nBlocks;        // Size (in bytes)
  UINT32 m_nCh;            // Number of channel which should be read 
} TTCAN_READ_DMA_CH_DATA_BLOCKS_IN;

//IOCTL_TTCANDEV_GET_STATUS
typedef struct _TTCAN_GET_STATUS_IN{
  UINT32 m_nTime;
} TTCAN_GET_STATUS_IN;

// IOCTL_TTCAN_WRITE_REG
typedef struct _TTCAN_WRITE_REG_IN{
  UINT32 m_nAddress;
  UINT32 m_nData;
} TTCAN_WRITE_REG_IN;

typedef struct _TTCAN_WRITE_REG_OUT{
  UINT32 m_nData;
} TTCAN_WRITE_REG_OUT;

//IOCTL_TTCAN_READ_REG
typedef struct _TTCAN_READ_REG_IN{
  UINT32 m_nAddress;
} TTCAN_READ_REG_IN;

typedef struct _TTCAN_READ_REG_OUT{
  UINT32 m_nData;
} TTCAN_READ_REG_OUT;

//IOCTL_TTCANDEV_DMA_COUNT
typedef struct _TTCAN_DMA_COUNT_OUT{
  UINT32 m_nData;
} TTCAN_DMA_COUNT_OUT;

//IOCTL_TTCANDEV_READ_DMA_STATUS
typedef struct _TTCAN_READ_DMA_STATUS_OUT{
  UINT32 m_nStatus;
} TTCAN_READ_DMA_STATUS_OUT;

//IOCTL_TTCANDEV_INT_FLAGS
typedef struct _TTCAN_INT_FLAGS_OUT{
  UINT32 m_nMainInt;
  UINT32 m_nCan1Int;
  UINT32 m_nCan2Int;
} TTCAN_INT_FLAGS_OUT;

// IOCTL_TTCAN_GET_RESOURCE_INFO
typedef struct _TTCAN_GET_RESOURCE_INFO_OUT{
  UINT32 m_nBus;
  UINT32 m_nSlot;
  UINT32 m_nFunction;
  UINT32 m_nDataBufPhysAddr;
  UINT32 m_nDataBufSize;
  UINT32 m_BarPhysAddr;
  UINT8  m_szCompileDate[128];
} TTCAN_GET_RESOURCE_INFO_OUT;

// IOCTL_TTCAN_READ_DATABUF
typedef struct _TTCAN_READ_DATABUF_IN{
  UINT32 m_nOffset;      // Offset of data (in bytes)
  UINT32 m_nSize;        // Size (in bytes)
} TTCAN_READ_DATABUF_IN;

// IOCTL_TTCANDEV_WRITE_DATABUF
typedef struct _TTCAN_WRITE_DATABUF_IN{
  UINT32 m_nOffset;      // Offset of data (in bytes)
  UINT32 m_nSize;        // Size (in bytes)
  UINT8 m_Data[0];       // array of bytes
} TTCAN_WRITE_DATABUF_IN;

// IOCTL_TTCAN_READ_BAR
typedef struct _TTCAN_READ_BAR_IN{
  UINT32 m_nBar;
  UINT32 m_nOffset;      // Offset of data (in bytes)
  UINT32 m_nSize;        // Size (in bytes)
} TTCAN_READ_BAR_IN;

// IOCTL_TTCAN_WRITE_BAR
typedef struct _TTCAN_WRITE_BAR_IN{
  UINT32 m_nBar;
  UINT32 m_nOffset;      // Offset of data (in bytes)
  UINT32 m_nSize;        // Size (in bytes)
  UINT8 m_Data[0];       // array of bytes
} TTCAN_WRITE_BAR_IN;

//IOCTL_TTCAN_REGISTER_EVENT
typedef struct _TTCAN_INT_NTFCN_IN{
	UINT8 m_nCh;
	UINT32 m_nInt;
	HANDLE m_hEvt;
}	TTCAN_INT_NTFCN_IN;

/////////////////////////////////////////////////////////////////////////////
// Macros
#define GetAbsoluteDecimation(ch, pch, out){                                                                               \
  if((ch == 0)                                                                  \
     || (pch[ch].m_nDdcDataSource == CAN_DDC_DATA_SOURCE_REALTIME)            \
     || (pch[0].m_nDdcCicDecimation == 0))                                      \
    out = pch[ch].m_nDdcCicDecimation;                                          \
  else                                                                          \
    out = pch[0].m_nDdcCicDecimation * pch[ch].m_nDdcCicDecimation;             \
  if(out == 0)                                                                  \
    out = 1;                                                                    \
}
#endif