
#ifndef _TTCAN_DEVICE_EX_H
#define _TTCAN_DEVICE_EX_H

#include "TTCANDevice.h"
#include "StdInterfaces.h"

#define BUF1	0
#define BUF2	1
#define BUF3	2

/////////////////////////////////////////////////////////////////////////////
// Class TTCCANDeviceEx

class TTCCANDeviceEx : public TTCCANDevice{
//CRITICAL_SECTION for threadSafe
private:
  CRITICAL_SECTION csRDDOB;
  CRITICAL_SECTION csSD;

public:
  TTCCANDeviceEx();
  virtual ~TTCCANDeviceEx();

  //Standart functions 
  BOOL   ReadBar(UINT32 nAddr, void *pData, UINT32 nSize);
  BOOL   ReadReg(UINT32 nAddr, UINT32 *pData);
  BOOL   WriteBar(UINT32 nAddr, const void *pData, UINT32 nSize);
  BOOL   WriteReg(UINT32 nAddr, UINT32 pData);

  //Configuration functions
  UINT32 WriteCanReg(UINT32 nChannel, UINT32 nAddress, UINT32 nSize, void *pDataBuf);
  UINT32 ReadCanReg(UINT32 nChannel, UINT32 nAddress, UINT32 nSize, void *pDataBuf); 
  UINT32 ModifyBit(UINT32 nChannel, UINT32 nAddress, UINT16 nData);
  UINT32 EnableDMA(UINT32 nCh);
  UINT32 DisableDMA(UINT32 nCh);
  UINT32 ResetChannel(UINT32 nChannel);
  UINT32 SetMode(UINT32 nChannel, UINT32 nMode); 
  UINT32 GetMode(UINT32 nChannel, UINT32 *nMode);
  UINT32 SetConfigurationMode(UINT32 nChannel);
  UINT32 SetLastModeBeforeConf(UINT32 nChannel);
  UINT32 ReadErrors(UINT32 nChannel, UINT8 *TEC_Value, UINT8 *REC_Value, UINT8 *EFLG_Value);
  UINT32 SetSpeed(UINT32 nChannel, UINT32 SpeedValue);
  UINT32 SetSpeedByParametrs(UINT32 nChannel, UINT8 BRP, UINT8 SJW, UINT8 SAM, UINT8 BTLMODE,
	                         UINT8 PHSEG1, UINT8 PHSEG2, UINT8 PRSEG, UINT8 WAKFIL);
  UINT32 SetABAT(UINT32 nChannel, BOOL bFlag);
  UINT32 SetReceiveMode(UINT32 nCh, UINT32 nBuf, UINT32 nMode);
  UINT32 SetRXFn(UINT32 nCh, UINT32 nFilterNum, UINT32 nSID, UINT32 nFlag, UINT32 nEID_or_nData2Bytes);
  UINT32 SetRXMn(UINT32 nCh, UINT32 nMaskNum, UINT32 nSID, UINT32 nFlag, UINT32 nEID_or_nData2Bytes);

  //Functions for work with data
  UINT32 SendData(UINT32 nChannel, UINT32 nBufNumber, UINT32 nPriority,
				  UINT32 SID, UINT32 EID, void *pData, UINT32 nSize);
  UINT32 WriteDataToSendBuffer(UINT32 nChannel, UINT32 nBufNumber,
							   UINT32 SID, UINT32 EID, void *pData, UINT32 nSize);
  UINT32 SendDataRightNow(UINT32 nChannel, UINT32 nPriority);
  UINT32 SendDataByTrigger(UINT32 nChannel, UINT32 nBufNumber, UINT32 nTrigger);
  UINT32 SendDataByTimer(UINT32 nChannel, UINT32 nBufNumber, UINT32 nTimer);
  UINT32 Set_CANn_TIMER_TRSH(UINT32 nChannel, UINT32 nValue);
  UINT32 Set_CANn_TIMER_CEED(UINT32 nChannel, UINT32 nValue);
  UINT32 SET_RST_ON_RXB1(UINT32 nChannel, bool flag);


  UINT32 ResetTrigger(UINT32 nChannel, UINT32 nBufNumber);
  UINT32 CheckTrigger(UINT32 nChannel, UINT32 nBufNumber);
  void   DataAnalysis(const void *pSourseData, UINT32 *nBuf, UINT32 *nCh, UINT32 *nTimer,
	                  UINT32 *nSID, UINT32 *nEID, void *OnlyData, UINT32 *nSize);
  UINT32 CheckTransmit(UINT32 nCh, UINT32 nBufNumber);
  UINT32 EndTransmit(UINT32 nCh, UINT32 nBufNumber);
  UINT32 ReadDMADataOneBlock(void *pData);
  BOOL   ReadDMADataBlocks(void *pData, UINT32 nBlocks, UINT32 *pBlocks);
  BOOL   GetNBlockRawDMA(UINT32 *nValue);
  BOOL   GetDMAReadyStatus(UINT32 *bFlag);
  BOOL   GetProgCount(UINT32 *nCount);
  BOOL   ClearDMA();

  //Canonical
  UINT32 ReadDMACanonicalDataBlocks(void *pData, UINT32 nCh, UINT32 nBlocks, UINT32 *pBlocks);
  BOOL   DisableCanonicalDMA();
  BOOL   EnableCanonicalDMA();

  //Interrupt
  UINT32 Register_Interrupt(UINT32 nCh, UINT32 InterruptName, HANDLE hEvent);
  UINT32 DisableInterrupt(UINT32 nCh, UINT32 InterruptName);

protected:
  BOOL    m_bExchangeStarted;
  BOOL    m_bBufferedIO;
  BOOL    m_bDelayedDownlink;

};

#endif
