// mPCIe-TTCAN_test.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <iostream>
#include "windows.h"
#include "TTCANDeviceEx.h"

using namespace std;

bool testGetDMASize(){
  TTCCANDeviceEx obj;
  UINT32 nSize;
  obj.Open(0);
  if (obj.GetDatabufSize(&nSize))
	cout<<hex<<"DMA data buf size = 0x"<<nSize<<endl;
  else
    cout<<"Error reading DMA data buf size"<<endl;
  obj.Close();
  return true;
}

bool testGetDmaAddress(){
  TTCCANDeviceEx obj;
  UINT32 nAddress;
  obj.Open(0);
  if (obj.GetDatabufPhysAddr(&nAddress))
	cout<<hex<<"DMA data buf address = 0x"<<nAddress<<endl;
  else
    cout<<"Error reading DMA data buf address"<<endl;
  obj.Close();
  return true;
}

bool testGetBarAddress(){
  TTCCANDeviceEx obj;
  UINT32 nAddress;
  obj.Open(0);
  if (obj.GetBarPhysAddr(&nAddress))
    cout<<hex<<"BAR address = 0x"<<nAddress<<endl;
  else
    cout<<"Error reading BAR address"<<endl;
  obj.Close();
  return true;
}

bool testGetPCILoc(){
  TTCCANDeviceEx obj;
  UINT32 nBus, nSlot;
  obj.Open(0);
  if (obj.GetPCILocation(&nBus, &nSlot))
    cout<<"BUS = "<<nBus<<", Slot = "<<nSlot<<endl;
  else
    cout<<"Error reading Bus and Slot address"<<endl;
  return true;
}

bool testGetRevision(){
  TTCCANDeviceEx objEx;
  UINT32 nRev;
  objEx.Open(0);
  if (objEx.GetRevision(&nRev))
    cout<<"Revesion = "<<nRev<<endl;
  else
    cout<<"Error reading revision"<<endl;
  objEx.Close();
  return true;
}

bool testGetDriverCompileDate(){
  TTCCANDeviceEx objEx;
  string sStr;
  objEx.Open(0);
  if (objEx.GetDriverCompileDate(&sStr))
	cout<<"Driver created: "<<sStr<<endl;
  else
    cout<<"Error reading data driver created"<<endl;
  objEx.Close();
  return true;
}

bool testWriteBar(){
  TTCCANDeviceEx objEx;
  UINT32 nAddr = 0x0, nData[2];
  nData[0] = 0x11223344;
  nData[1] = 0x55667788;
  objEx.Open(0);
  if (objEx.WriteBar(nAddr, &nData, 8))
    cout<<hex<<"Data was written to 0x"<<nAddr<<" address."<<endl;
  else
    cout<<"Error writing bar"<<endl;
  objEx.Close();
return true;
}

bool testReadBar(){
  TTCCANDeviceEx objEx;
  UINT32 pData[2], nAddr = 0x0;
  objEx.Open(0);
  if (objEx.ReadBar(nAddr, &pData, 8)){
	  cout<<"From 0x"<<nAddr<<" address was read data: "<<endl;
	  cout<<hex<<"0x"<<pData[0]<<endl;
	  cout<<hex<<"0x"<<pData[1]<<endl;
  }
  else
    cout<<"Error reading bar"<<endl;
  objEx.Close();
  return true;
}

bool testResetChannel(UINT32 nCh){
  TTCCANDeviceEx objEx;
  objEx.Open(0);
  if (objEx.ResetChannel(nCh)!= 1){
	  cout<<"Error resetting ch "<<nCh<<endl;
	  return false;
  }
  else
    cout<<nCh<<"Ch was resetted"<<endl;
    
  objEx.Close();
  return true;
}

void init(bool bTrigger){
	TTCCANDeviceEx Device;
	int nErr,
		nSpeedCh1=125,
		nSpeedCh2=125;

	// 
    if (!Device.Open(0)){
        cout<<"Can't open device\n";
        return;
    }

    // 
    if ((nErr = Device.SetMode(1, 4)) != 1){
        cout<<"Can't set conf work mode Ch1. Err = "<<nErr<<endl;
        return;
    }
    if ((nErr = Device.SetMode(2, 4)) != 1){
        cout<<"Can't set conf work mode Ch2. Err = "<<nErr<<endl;
        return;
    }

	if (bTrigger){
		//    
		int var1=0x07,
			var2=0;
		if ((nErr = Device.WriteCanReg(1, 0x0D, 1, &var1)) != 1){
			cout<<"Ch1. Can't enable triggers. Err = "<<nErr<<endl;
			return;
		}

		if ((nErr = Device.WriteCanReg(2, 0x0D, 1, &var1)) != 1){
			cout<<"Ch2. Can't enable triggers. Err = "<<nErr<<endl;
			return;
		}
	}

    //
    if (!(Device.SetSpeed(1, nSpeedCh1))){
        cout<<"Can't set speed Ch1. Err = "<<nErr<<endl;
        return;
    }
    if (!(Device.SetSpeed(2, nSpeedCh2))){
        cout<<"Can't set speed Ch2. Err = "<<nErr<<endl;
        return;
    }

    // 
    if ((nErr = Device.SetMode(1, 0)) != 1){
        cout<<"Can't set conf work mode Ch1. Err = "<<nErr<<endl;
        return;
    }
    if ((nErr = Device.SetMode(2, 0)) != 1){
        cout<<"Can't set conf work mode Ch2. Err = "<<nErr<<endl;
        return;
    }

    // DMA
    if ((nErr = Device.EnableDMA(0)) !=1){
        cout<<"Can't enable DMA. Err = "<<nErr<<endl;
        return;
    }

    // /  
	if ((nErr = Device.SetReceiveMode(1, 0, 3)) != 1){
        cout<<"Can't unset filters. Err = "<<nErr<<endl;
        return;
    }
	if ((nErr = Device.SetReceiveMode(2, 0, 3)) != 1){
        cout<<"Can't unset filters. Err = "<<nErr<<endl;
        return;
    }

	Device.Close();

	cout<<"Module initizlization was success"<<endl;
}

bool checkTransmit(TTCCANDeviceEx *Device, UINT32 nCh, UINT32 nBuf){
    int nRes;
    bool bFlag=false;

    for (int a=0; a<2500; a++){
        //   
        if ((nRes = Device->CheckTransmit(nCh, nBuf)) == 1){
            bFlag = true;
            break;
        }
    }

    //  
    if (!bFlag){
        cout<<"CheckTransmit I/O error. Code = "<<nRes<<endl;
        // 
      if((nRes = Device->EndTransmit(nCh,nBuf)) != 1){
          cout<<"Can't end transmit. Code = "<<nRes<<endl;
      }
      return false;
    }

    return true;
}

bool checkTransmit_TT(TTCCANDeviceEx *Device, UINT32 nCh, UINT32 nBuf){
    int nRes=0;
    bool bFlag=false;

    for (int a=0; a<2500; a++){
		//cout<<"a = "<<a<<endl;
		Sleep(100);
        //   
        if ((nRes = Device->CheckTrigger(nCh, nBuf)) == 1){
            bFlag = true;
            break;
        }
    }

    //  
    if (!bFlag){
        cout<<"CheckTransmit I/O error. Code = "<<nRes<<endl;
        // 
		if((nRes = Device->ResetTrigger(nCh,nBuf)) != 1){
          cout<<"Can't end transmit. Code = "<<nRes<<endl;
      }
      return false;
    }

    return true;
}

void sendData_Old(){
	TTCCANDeviceEx Device;
int	nRes,
	nSID=0xAB,
	nEID=0xBA,
	nSumCh1=0,
	sendbuff1=41,
	sendbuff2=44;

// 
    if (!Device.Open(0)){
        cout<<"Can't open device\n";
        return;
    }

 if ((nRes = Device.SendData(1, 0, 0, nSID, nEID, &sendbuff1, 4)) != 1){
	cout<<"SendData I/O error. Code = "<<nRes;
	return;
  }

	//    
	if (!checkTransmit(&Device, 1, 0)){
		cout<<"check transmit error 1 : 0"<<endl;
		return;
	}
	if ((nRes = Device.SendData(1, 1, 0, nSID, nEID, &sendbuff2, 4)) != 1){
	cout<<"SendData I/O error. Code = "<<nRes;
	return;
  }

	//    
	if (!checkTransmit(&Device, 1, 1)){
		cout<<"check transmit error 1 : 1"<<endl;
		return;
	}

	Device.Close();

	cout<<"Data has been successfuly send."<<endl;
}

void sendData(){
	TTCCANDeviceEx Device;
	int nRes;

	// 
    if (!Device.Open(0)){
        cout<<"Can't open device\n";
        return;
    }

	int nCh=2,
		nSID=0xAB,
		nEID=0xBA,
		nSumCh1=0,
		sendbuff[4];
    sendbuff[0]=41;
    sendbuff[1]=18467;
    sendbuff[2]=6334;
    sendbuff[3]=1930;

	 if (!Device.ClearDMA()){
        cout<< "Send error: Can't clear DMA\n";
        return;
    }

	// 3      
    if ((nRes = Device.WriteDataToSendBuffer(nCh, BUF1, nSID, nEID, &sendbuff[0], 4)) != 1){
        cout<<"WriteDataToSendBuffer I/O error. Code = "<<nRes<<endl;
        return;
    }
	if ((nRes = Device.SendDataRightNow(nCh, BUF1)) != 1){
        cout<<"SendDataRightNow I/O error. Code = "<<nRes<<endl;
        return;
    }
    if ((nRes = Device.WriteDataToSendBuffer(nCh, BUF2, nSID, nEID, &sendbuff[1], 4)) != 1){
        cout<<"WriteDataToSendBuffer I/O error. Code = "<<nRes<<endl;
        return;
    }
	if ((nRes = Device.SendDataRightNow(nCh, BUF2)) != 1){
        cout<<"SendDataRightNow I/O error. Code = "<<nRes<<endl;
        return;
    }
    if ((nRes = Device.WriteDataToSendBuffer(nCh, BUF3, nSID, nEID, &sendbuff[2], 4)) != 1){
        cout<<"WriteDataToSendBuffer I/O error. Code = "<<nRes<<endl;
        return;
    }
	if ((nRes = Device.SendDataRightNow(nCh, BUF3)) != 1){
        cout<<"SendDataRightNow I/O error. Code = "<<nRes<<endl;
        return;
    }
    //    
     if (!checkTransmit(&Device, nCh, BUF1)){
        cout<<"check transmit error 1 : 0"<<endl;
        return;
    }

    //   
    if ((nRes = Device.WriteDataToSendBuffer(nCh, BUF1, nSID, nEID, &sendbuff[3], 4)) != 1){
        cout<<"WriteDataToSendBuffer I/O error. Code = "<<nRes<<endl;
        return;
    }
	if ((nRes = Device.SendDataRightNow(nCh, BUF1)) != 1){
        cout<<"SendDataRightNow I/O error. Code = "<<nRes<<endl;
        return;
    }

    //   4 
    for (int i=0; i<4; i++)
        nSumCh1+=sendbuff[i];
    //    
    if (!checkTransmit(&Device, nCh, BUF2)){
        cout<<"check transmit error 1 : 1"<<endl;
        return;
    }

    //   
    if ((nRes = Device.WriteDataToSendBuffer(nCh, BUF2, nSID, nEID, &nSumCh1, 4)) != 1){
        cout<<"WriteDataToSendBuffer I/O error. Code = "<<nRes<<endl;
        return;
    }
	if ((nRes = Device.SendDataRightNow(nCh, BUF2)) != 1){
        cout<<"SendDataRightNow I/O error. Code = "<<nRes<<endl;
        return;
    }

    //      
    if (!checkTransmit(&Device, nCh, BUF3)){
        cout<<"check transmit error 1 : 2"<<endl;
        return;
    }

    //      
    if (!checkTransmit(&Device, nCh, BUF1)){
        cout<<"check transmit error 1 : 0"<<endl;
        return;
    }

    //      
    if (!checkTransmit(&Device, nCh, BUF2)){
        cout<<"check transmit error 1 : 1"<<endl;
        return;
    }

	Device.Close();

	cout<<"Data has been successfuly send."<<endl;
}

void sendDataByTrigger(){
	TTCCANDeviceEx Device;
	int nRes;

	// 
    if (!Device.Open(0)){
        cout<<"Can't open device\n";
        return;
    }

	UINT32 nTemp,
		   nTimer;

	int nSID=0xAB,
		nEID=0xBA,
		nSumCh1=0,
		sendbuff[4];
    sendbuff[0]=41;
    sendbuff[1]=18467;
    sendbuff[2]=6334;
    sendbuff[3]=1930;


	 if (!Device.ClearDMA()){
        cout<< "Send error: Can't clear DMA\n";
        return;
    }
	 //  
	 if (!Device.ReadReg(0x1060, &nTimer)){
		cout<< "ReadReg error.\n";
        return;
	 }

	 //cout<<hex<<"Timer = "<<nTimer<<endl;

	nTemp = (4<<16);
	//cout<<hex<<"2s = "<<nTemp<<endl;
	nTemp += nTimer;
	//cout<<hex<<"Timer + 2s = "<<nTemp<<endl;

	 // 3      
    if ((nRes = Device.WriteDataToSendBuffer(1, BUF1, nSID, nEID, &sendbuff[0], 4)) != 1){
        cout<<"WriteDataToSendBuffer I/O error. Code = "<<nRes<<endl;
        return;
    }
	if ((nRes = Device.SendDataByTrigger(1, BUF1, nTemp)) != 1){
        cout<<"SendDataByTrigger I/O error. Code = "<<nRes<<endl;
        return;
    }

	nTemp = (5<<16);
	nTemp += nTimer;
    if ((nRes = Device.WriteDataToSendBuffer(1, BUF2, nSID, nEID, &sendbuff[1], 4)) != 1){
        cout<<"WriteDataToSendBuffer I/O error. Code = "<<nRes<<endl;
        return;
    }
	if ((nRes = Device.SendDataByTrigger(1, BUF2, nTemp)) != 1){
        cout<<"SendDataByTrigger I/O error. Code = "<<nRes<<endl;
        return;
    }

	nTemp = (6<<16);
	nTemp += nTimer;
    if ((nRes = Device.WriteDataToSendBuffer(1, BUF3, nSID, nEID, &sendbuff[2], 4)) != 1){
        cout<<"WriteDataToSendBuffer I/O error. Code = "<<nRes<<endl;
        return;
    }
	if ((nRes = Device.SendDataByTrigger(1, BUF3, nTemp)) != 1){
        cout<<"SendDataByTrigger I/O error. Code = "<<nRes<<endl;
        return;
    }

    //    
    if (!checkTransmit_TT(&Device, 1, BUF1)){
        cout<<"check transmit error 1 : 0"<<endl;
        return;
    }

	//  
	 if (!Device.ReadReg(0x1060, &nTimer)){
		cout<< "ReadReg error.\n";
        return;
	 }
    //   
	nTemp = (2<<16);
	nTemp += nTimer;
    if ((nRes = Device.WriteDataToSendBuffer(1, BUF1, nSID, nEID, &sendbuff[3], 4)) != 1){
        cout<<"WriteDataToSendBuffer I/O error. Code = "<<nRes<<endl;
        return;
    }
	if ((nRes = Device.SendDataByTrigger(1, BUF1, nTemp)) != 1){
        cout<<"SendDataByTrigger I/O error. Code = "<<nRes<<endl;
        return;
    }

    //   4 
    for (int i=0; i<4; i++)
        nSumCh1+=sendbuff[i];
    //    
    if (!checkTransmit_TT(&Device, 1, BUF2)){
        cout<<"check transmit error 1 : 1"<<endl;
        return;
    }

	//  
	 if (!Device.ReadReg(0x1060, &nTimer)){
		cout<< "ReadReg error.\n";
        return;
	 }
    //   
	nTemp = (2<<16);
	nTemp += nTimer;
    if ((nRes = Device.WriteDataToSendBuffer(1, BUF2, nSID, nEID, &nSumCh1, 4)) != 1){
        cout<<"WriteDataToSendBuffer I/O error. Code = "<<nRes<<endl;
        return;
    }
	if ((nRes = Device.SendDataByTrigger(1, BUF2, nTemp)) != 1){
        cout<<"SendDataByTrigger I/O error. Code = "<<nRes<<endl;
        return;
    }

    //      
    if (!checkTransmit_TT(&Device, 1, BUF3)){
        cout<<"check transmit error 1 : 2"<<endl;
        return;
    }

    //      
    if (!checkTransmit_TT(&Device, 1, BUF1)){
        cout<<"check transmit error 1 : 0"<<endl;
        return;
    }

    //      
    if (!checkTransmit_TT(&Device, 1, BUF2)){
        cout<<"check transmit error 1 : 1"<<endl;
        return;
    }

	Device.Close();

	cout<<"Data has been successfuly send."<<endl;

}

void readData(){
	TTCCANDeviceEx Device;
	//int nRes;
	UINT32	nBlocks;

	// 
    if (!Device.Open(0)){
        cout<<"Can't open device\n";
        return;
    }

	//if (!Device.EnableCanonicalDMA()){
      //  qDebug() << "Receive error: Can't set canonical DMA";
       // return;
    //}

	if(!Device.GetNBlockRawDMA(&nBlocks)){
		cout<<"error GetNBlockRawDMA"<<endl;
		return;
    }
	else{
		cout<<"There is "<<nBlocks<<" blocks of new data"<<endl;
	}

	if (nBlocks>0){
		UINT32 pData[16], pBlocks;
		if (Device.ReadDMADataBlocks(&pData, 1, &pBlocks) != 1){
			cout<<"Error reading dma data"<<endl;
			return;
		}
		cout<<"Data:"<<endl;
		for (int i=0; i<16; i++)
				cout<<hex<<"0x"<<pData[i]<<endl;
		UINT32 nBuf, nCh, nTimer, nSID, nEID, OnlyData, nSize;
		Device.DataAnalysis(&pData, &nBuf, &nCh, &nTimer, &nSID, &nEID, &OnlyData, &nSize);
		cout<<"DMA data block info:"<<endl;
		cout<<hex<<"nCh =0x"<<nCh<<". nBuf =0x"<<nBuf<<".\nnTimer =0x"<<nTimer<<".\nnSID =0x"<<nSID<<". nEID =0x"<<nEID<<". Data size = 0x"<<nSize<<endl;
		if (nSize){
			cout<<"Data = 0x"<<OnlyData<<endl;
		}
	}

	Device.Close();
}

void SendBytriggerLoop(){
	TTCCANDeviceEx Device;
	int nRes;

	// 
    if (!Device.Open(0)){
        cout<<"Can't open device\n";
        return;
    }

	int nSID=0xAB,
		nEID=0xBA,
		sendbuff = 41;


	 /*if (!Device.ClearDMA()){
        cout<< "Send error: Can't clear DMA\n";
        return;
    }*/

	 //    
    if ((nRes = Device.WriteDataToSendBuffer(1, BUF1, nSID, nEID, &sendbuff, 4)) != 1){
        cout<<"WriteDataToSendBuffer I/O error. Code = "<<nRes<<endl;
        return;
    }
	if ((nRes = Device.Set_CANn_TIMER_TRSH(1, 0x10FFFC)) != 1){
        cout<<"Set_CANn_TIMER_TRSH I/O error. Code = "<<nRes<<endl;
        return;
    }
	if ((nRes = Device.SendDataByTimer(1, BUF1, 100)) != 1){
        cout<<"SendDataByTimer I/O error. Code = "<<nRes<<endl;
        return;
    }

	Sleep(20000);
	if ((nRes = Device.ResetTrigger(1, BUF1)) != 1){
        cout<<"ResetTrigger I/O error. Code = "<<nRes<<endl;
        return;
    }

	Device.Close();

	cout<<"Data has been successfuly send."<<endl;

}

bool testReadReg(UINT32 nAddr){
  TTCCANDeviceEx objEx;
  UINT32 pData;
  objEx.Open(0);
  if (objEx.ReadReg(nAddr, &pData))
    cout<<hex<<"From 0x"<<nAddr<<" reg was read "<<pData<<"\n";
  else
    cout<<"I/O error\n";
  objEx.Close();
  return true;
}

bool testReadCanReg(UINT32 nCh, UINT32 nAddr, UINT32 nSize){
  TTCCANDeviceEx objEx;
  objEx.Open(0);
  UINT16 pDataBuf, nErr;
  nErr = objEx.ReadCanReg(nCh, nAddr, nSize, &pDataBuf);
  if (nErr == 1)
    cout<<hex<<nCh<<" Ch from 0x"<<nAddr<<" reg was read 0x"<<pDataBuf<<"\n"; 
  else
	cout<<"I/O error\n";
	objEx.Close();
return true;
}

bool testWriteReg(UINT32 nAddr, UINT32 nData){
	TTCCANDeviceEx objEx;
	objEx.Open(0);
	if (objEx.WriteReg(nAddr, nData))
		cout<<hex<<"Value 0x"<<nData<<" was written to 0x"<<nAddr<<" reg\n";
	else
		cout<<"I/O error";
	objEx.Close();
return true;
}

bool testWriteCanReg(UINT32 nCh, UINT32 nAddr, UINT8 pData){
  TTCCANDeviceEx objEx;
  objEx.Open(0);
  UINT32 nErr=0, nSize=1;
  nErr = objEx.WriteCanReg(nCh, nAddr, nSize, &pData);

  if (nErr == 1)
    cout<<hex<<nCh<<" Ch value 0x"<<pData<<" was written to 0x"<<nAddr<<" reg\n";
  else
    cout<<"I/O error";
  objEx.Close();
  return true;
}

bool testSetRST_ON_RXB1(UINT32 nCh, bool flag){
  TTCCANDeviceEx objEx;
  objEx.Open(0);

  if (objEx.SET_RST_ON_RXB1(nCh, flag)==1)
	  cout<<"RXB was sucessfully set/unset."<<endl;
  else
    cout<<"I/O error";

  objEx.Close();
  return true;
}

bool testSet_CANn_TIMER_CEED(UINT32 nCh, UINT32 nValue){
  TTCCANDeviceEx objEx;
  objEx.Open(0);

  if (objEx.Set_CANn_TIMER_CEED(nCh, nValue)==1)
	  cout<<"CANn_TIMER_CEED was sucessfully set."<<endl;
  else
    cout<<"I/O error";

  objEx.Close();
  return true;
}

bool testSet_CANn_TIMER_TRSH(UINT32 nCh, UINT32 nValue){
  TTCCANDeviceEx objEx;
  objEx.Open(0);

  if (objEx.Set_CANn_TIMER_TRSH(nCh, nValue)==1)
	  cout<<"Set_CANn_TIMER_TRSH was sucessfully set."<<endl;
  else
    cout<<"I/O error";

  objEx.Close();
  return true;
}

void info(){
	testGetDMASize();
	testGetDmaAddress();
	testGetBarAddress();
	testGetPCILoc();
	testGetRevision();
	testGetDriverCompileDate();
}

int _tmain(int argc, _TCHAR* argv[]){
	
	//info();
	
	//testWriteBar();
	//testReadBar();
	//testResetChannel(2);
	//testSetRST_ON_RXB1(2, false);
	testSet_CANn_TIMER_TRSH(2, 0x1234);
	testSet_CANn_TIMER_CEED(2, 0x4321);


	//init(true);
	//sendData_Old();
	//sendData();
	//sendDataByTrigger();
	//SendBytriggerLoop();
	//readData();
	

	
	cin.get(); cin.get();
	return 0;
}

